@@ -36,15 +36,6 @@ typedef struct {
         DhLink        *selected_link;
 } DhBookTreePrivate;
 
-static void book_tree_add_columns          (DhBookTree       *tree);
-static void book_tree_setup_selection      (DhBookTree       *tree);
-static void book_tree_insert_node          (DhBookTree       *tree,
-                                            GNode            *node,
-                                            GtkTreeIter      *current_iter,
-                                            DhBook           *book);
-static void book_tree_selection_changed_cb (GtkTreeSelection *selection,
-                                            DhBookTree       *tree);
-
 enum {
         LINK_SELECTED,
         LAST_SIGNAL
@@ -91,6 +82,7 @@ dh_book_tree_get_property (GObject    *object,
                 case PROP_BOOK_MANAGER:
                         g_value_set_object (value, priv->book_manager);
                         break;
+
                 default:
                         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                         break;
@@ -110,36 +102,13 @@ dh_book_tree_set_property (GObject      *object,
                         g_return_if_fail (priv->book_manager == NULL);
                         priv->book_manager = g_value_dup_object (value);
                         break;
+
                 default:
                         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                         break;
         }
 }
 
-static void
-dh_book_tree_init (DhBookTree *tree)
-{
-        DhBookTreePrivate *priv;
-
-        priv = dh_book_tree_get_instance_private (tree);
-
-        priv->store = gtk_tree_store_new (N_COLUMNS,
-                                          G_TYPE_STRING,
-                                          G_TYPE_POINTER,
-                                          G_TYPE_OBJECT,
-                                          PANGO_TYPE_WEIGHT,
-                                          PANGO_TYPE_UNDERLINE);
-        priv->selected_link = NULL;
-        gtk_tree_view_set_model (GTK_TREE_VIEW (tree),
-                                 GTK_TREE_MODEL (priv->store));
-
-        gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (tree), FALSE);
-
-        book_tree_add_columns (tree);
-
-        book_tree_setup_selection (tree);
-}
-
 static void
 book_tree_add_columns (DhBookTree *tree)
 {
@@ -162,6 +131,29 @@ book_tree_add_columns (DhBookTree *tree)
         gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
 }
 
+static void
+book_tree_selection_changed_cb (GtkTreeSelection *selection,
+                                DhBookTree       *tree)
+{
+        DhBookTreePrivate *priv = dh_book_tree_get_instance_private (tree);
+        GtkTreeIter     iter;
+
+        if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
+                DhLink *link;
+
+                gtk_tree_model_get (GTK_TREE_MODEL (priv->store),
+                                    &iter,
+                                    COL_LINK, &link,
+                                    -1);
+
+                if (link != NULL &&
+                    link != priv->selected_link) {
+                        priv->selected_link = link;
+                        g_signal_emit (tree, signals[LINK_SELECTED], 0, link);
+                }
+        }
+}
+
 static void
 book_tree_setup_selection (DhBookTree *tree)
 {
@@ -178,6 +170,29 @@ book_tree_setup_selection (DhBookTree *tree)
                                  0);
 }
 
+static void
+dh_book_tree_init (DhBookTree *tree)
+{
+        DhBookTreePrivate *priv;
+
+        priv = dh_book_tree_get_instance_private (tree);
+
+        priv->store = gtk_tree_store_new (N_COLUMNS,
+                                          G_TYPE_STRING,
+                                          G_TYPE_POINTER,
+                                          G_TYPE_OBJECT,
+                                          PANGO_TYPE_WEIGHT,
+                                          PANGO_TYPE_UNDERLINE);
+        priv->selected_link = NULL;
+        gtk_tree_view_set_model (GTK_TREE_VIEW (tree),
+                                 GTK_TREE_MODEL (priv->store));
+
+        gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (tree), FALSE);
+
+        book_tree_add_columns (tree);
+        book_tree_setup_selection (tree);
+}
+
 /* Tries to find:
  *  - An exact match of the language group
  *  - Or the language group which should be just after our given language group.
@@ -194,12 +209,13 @@ book_tree_find_language_group (DhBookTree  *tree,
         DhBookTreePrivate *priv = dh_book_tree_get_instance_private (tree);
         GtkTreeIter     loop_iter;
 
-        g_assert ((exact_iter && exact_found) || (next_iter && next_found));
+        g_assert ((exact_iter != NULL && exact_found != NULL) ||
+                  (next_iter != NULL && next_found != NULL));
 
         /* Reset all flags to not found */
-        if (exact_found)
+        if (exact_found != NULL)
                 *exact_found = FALSE;
-        if (next_found)
+        if (next_found != NULL)
                 *next_found = FALSE;
 
         /* If we're not doing language grouping, return not found */
@@ -214,25 +230,33 @@ book_tree_find_language_group (DhBookTree  *tree,
 
         do {
                 gchar *title = NULL;
+                DhLink *link;
 
                 /* Look for language titles, which are those where there
                  * is no book object associated in the row */
                 gtk_tree_model_get (GTK_TREE_MODEL (priv->store),
                                     &loop_iter,
                                     COL_TITLE, &title,
+                                    COL_LINK, &link,
                                     -1);
 
-                if (exact_iter &&
+                if (link != NULL) {
+                        /* Not a language */
+                        g_free (title);
+                        g_return_if_reached ();
+                }
+
+                if (exact_iter != NULL &&
                     g_ascii_strcasecmp (title, language) == 0) {
                         /* Exact match found! */
                         *exact_iter = loop_iter;
                         *exact_found = TRUE;
-                        if (!next_iter) {
+                        if (next_iter == NULL) {
                                 /* If we were not requested to look for the next one, end here */
                                 g_free (title);
                                 return;
                         }
-                } else if (next_iter &&
+                } else if (next_iter != NULL &&
                            g_ascii_strcasecmp (title, language) > 0) {
                         *next_iter = loop_iter;
                         *next_found = TRUE;
@@ -264,16 +288,17 @@ book_tree_find_book (DhBookTree        *tree,
         DhBookTreePrivate *priv = dh_book_tree_get_instance_private (tree);
         GtkTreeIter     loop_iter;
 
-        g_assert ((exact_iter && exact_found) || (next_iter && next_found));
+        g_assert ((exact_iter != NULL && exact_found != NULL) ||
+                  (next_iter != NULL && next_found != NULL));
 
         /* Reset all flags to not found */
-        if (exact_found)
+        if (exact_found != NULL)
                 *exact_found = FALSE;
-        if (next_found)
+        if (next_found != NULL)
                 *next_found = FALSE;
 
         /* Setup iteration start */
-        if (!first) {
+        if (first == NULL) {
                 /* If no first given, start iterating from the start of the model */
                 if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->store),
                                                     &loop_iter)) {
@@ -292,18 +317,20 @@ book_tree_find_book (DhBookTree        *tree,
                                     COL_BOOK, &in_tree_book,
                                     -1);
 
+                g_return_if_fail (DH_IS_BOOK (in_tree_book));
+
                 /* We can compare pointers directly as we're playing with references
                  * of the same object */
-                if (exact_iter &&
+                if (exact_iter != NULL &&
                     in_tree_book == book) {
                         *exact_iter = loop_iter;
                         *exact_found = TRUE;
-                        if (!next_iter) {
+                        if (next_iter == NULL) {
                                 /* If we were not requested to look for the next one, end here */
                                 g_object_unref (in_tree_book);
                                 return;
                         }
-                } else if (next_iter &&
+                } else if (next_iter != NULL &&
                            dh_book_cmp_by_title (in_tree_book, book) > 0) {
                         *next_iter = loop_iter;
                         *next_found = TRUE;
@@ -311,22 +338,61 @@ book_tree_find_book (DhBookTree        *tree,
                         return;
                 }
 
-                if (in_tree_book)
-                        g_object_unref (in_tree_book);
+                g_object_unref (in_tree_book);
         } while (gtk_tree_model_iter_next (GTK_TREE_MODEL (priv->store),
                                            &loop_iter));
 }
 
+static void
+book_tree_insert_node (DhBookTree  *tree,
+                       GNode       *node,
+                       GtkTreeIter *current_iter,
+                       DhBook      *book)
+
+{
+        DhBookTreePrivate *priv = dh_book_tree_get_instance_private (tree);
+        DhLink         *link;
+        PangoWeight     weight;
+        GNode          *child;
+
+        link = node->data;
+        g_assert (link != NULL);
+
+        if (dh_link_get_link_type (link) == DH_LINK_TYPE_BOOK) {
+                weight = PANGO_WEIGHT_BOLD;
+        } else {
+                weight = PANGO_WEIGHT_NORMAL;
+        }
+
+        gtk_tree_store_set (priv->store,
+                            current_iter,
+                            COL_TITLE, dh_link_get_name (link),
+                            COL_LINK, link,
+                            COL_BOOK, book,
+                            COL_WEIGHT, weight,
+                            COL_UNDERLINE, PANGO_UNDERLINE_NONE,
+                            -1);
+
+        for (child = g_node_first_child (node);
+             child != NULL;
+             child = g_node_next_sibling (child)) {
+                GtkTreeIter iter;
+
+                /* Append new iter */
+                gtk_tree_store_append (priv->store, &iter, current_iter);
+                book_tree_insert_node (tree, child, &iter, NULL);
+        }
+}
+
 static void
 book_tree_add_book_to_store (DhBookTree *tree,
-                             DhBook     *book,
-                             gboolean    group_by_language)
+                             DhBook     *book)
 {
         DhBookTreePrivate *priv = dh_book_tree_get_instance_private (tree);
         GtkTreeIter     book_iter;
 
         /* If grouping by language we need to add the language categories */
-        if (group_by_language) {
+        if (dh_book_manager_get_group_by_language (priv->book_manager)) {
                 GtkTreeIter  language_iter;
                 gboolean     language_iter_found;
                 GtkTreeIter  next_language_iter;
@@ -401,6 +467,7 @@ book_tree_add_book_to_store (DhBookTree *tree,
                                              NULL,
                                              &next_book_iter,
                                              &next_book_iter_found);
+
                         if (!next_book_iter_found) {
                                 gtk_tree_store_append (priv->store,
                                                        &book_iter,
@@ -424,6 +491,7 @@ book_tree_add_book_to_store (DhBookTree *tree,
                                      NULL,
                                      &next_book_iter,
                                      &next_book_iter_found);
+
                 if (!next_book_iter_found) {
                         gtk_tree_store_append (priv->store,
                                                &book_iter,
@@ -444,57 +512,106 @@ book_tree_add_book_to_store (DhBookTree *tree,
                                book);
 }
 
+static void
+book_tree_init_selection (DhBookTree *tree)
+{
+        DhBookTreePrivate   *priv;
+        GtkTreeSelection *selection;
+        GtkTreeIter       iter;
+        gboolean          iter_found = FALSE;
+
+        priv = dh_book_tree_get_instance_private (tree);
+
+        /* Mark the first item as selected, or it would get automatically
+         * selected when the treeview will get focus; but that's not even
+         * enough as a selection changed would still be emitted when there
+         * is no change, hence the manual tracking of selection in
+         * selected_link.
+         *   https://bugzilla.gnome.org/show_bug.cgi?id=492206
+         */
+        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
+        g_signal_handlers_block_by_func (selection,
+                                         book_tree_selection_changed_cb,
+                                         tree);
+
+        /* If grouping by languages, get first book in the first language */
+        if (dh_book_manager_get_group_by_language (priv->book_manager)) {
+                GtkTreeIter language_iter;
+
+                if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->store),
+                                                   &language_iter)) {
+                        iter_found = gtk_tree_model_iter_children (GTK_TREE_MODEL (priv->store),
+                                                                   &iter,
+                                                                   &language_iter);
+                }
+        } else {
+                iter_found = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->store),
+                                                            &iter);
+        }
+
+        if (iter_found) {
+                DhLink *link;
+
+                gtk_tree_model_get (GTK_TREE_MODEL (priv->store),
+                                    &iter,
+                                    COL_LINK, &link,
+                                    -1);
+
+                priv->selected_link = link;
+                gtk_tree_selection_select_iter (selection, &iter);
+
+                if (dh_link_get_link_type (link) != DH_LINK_TYPE_BOOK)
+                        g_warn_if_reached ();
+        }
+
+        g_signal_handlers_unblock_by_func (selection,
+                                           book_tree_selection_changed_cb,
+                                           tree);
+}
+
 static void
 book_tree_populate_tree (DhBookTree *tree)
 {
         DhBookTreePrivate *priv = dh_book_tree_get_instance_private (tree);
-        GList          *l;
-        gboolean        group_by_language;
-
-        group_by_language = dh_book_manager_get_group_by_language (priv->book_manager);
+        GList *l;
 
+        gtk_tree_view_set_model (GTK_TREE_VIEW (tree), NULL);
         gtk_tree_store_clear (priv->store);
+        gtk_tree_view_set_model (GTK_TREE_VIEW (tree),
+                                 GTK_TREE_MODEL (priv->store));
 
         /* This list comes in order, but we don't really mind */
         for (l = dh_book_manager_get_books (priv->book_manager);
-             l;
-             l = g_list_next (l)) {
+             l != NULL;
+             l = l->next) {
                 DhBook *book = DH_BOOK (l->data);
 
                 /* Only add enabled books to the tree */
-                if (dh_book_get_enabled (book)) {
-                        book_tree_add_book_to_store (tree,
-                                                     book,
-                                                     group_by_language);
-                }
+                if (dh_book_get_enabled (book))
+                        book_tree_add_book_to_store (tree, book);
         }
+
+        book_tree_init_selection (tree);
 }
 
 static void
 book_tree_book_created_or_enabled_cb (DhBookManager *book_manager,
                                       GObject       *book_object,
-                                      gpointer       user_data)
+                                      DhBookTree    *tree)
 {
-        DhBookTree     *tree = user_data;
-        DhBook         *book = DH_BOOK (book_object);
-        gboolean        group_by_language;
+        DhBook *book = DH_BOOK (book_object);
 
         if (!dh_book_get_enabled (book))
                 return;
 
-        group_by_language = dh_book_manager_get_group_by_language (book_manager);
-
-        book_tree_add_book_to_store (tree,
-                                     book,
-                                     group_by_language);
+        book_tree_add_book_to_store (tree, book);
 }
 
 static void
 book_tree_book_deleted_or_disabled_cb (DhBookManager *book_manager,
                                        GObject       *book_object,
-                                       gpointer       user_data)
+                                       DhBookTree    *tree)
 {
-        DhBookTree     *tree = user_data;
         DhBookTreePrivate *priv = dh_book_tree_get_instance_private (tree);
         DhBook         *book = DH_BOOK (book_object);
         GtkTreeIter     exact_iter;
@@ -552,127 +669,12 @@ book_tree_book_deleted_or_disabled_cb (DhBookManager *book_manager,
         }
 }
 
-static void
-book_tree_insert_node (DhBookTree  *tree,
-                       GNode       *node,
-                       GtkTreeIter *current_iter,
-                       DhBook      *book)
-
-{
-        DhBookTreePrivate *priv = dh_book_tree_get_instance_private (tree);
-        DhLink         *link;
-        PangoWeight     weight;
-        GNode          *child;
-
-        link = node->data;
-
-        if (dh_link_get_link_type (link) == DH_LINK_TYPE_BOOK) {
-                weight = PANGO_WEIGHT_BOLD;
-        } else {
-                weight = PANGO_WEIGHT_NORMAL;
-        }
-
-        gtk_tree_store_set (priv->store,
-                            current_iter,
-                            COL_TITLE, dh_link_get_name (link),
-                            COL_LINK, link,
-                            COL_BOOK, book,
-                            COL_WEIGHT, weight,
-                            COL_UNDERLINE, PANGO_UNDERLINE_NONE,
-                            -1);
-
-        for (child = g_node_first_child (node);
-             child;
-             child = g_node_next_sibling (child)) {
-                GtkTreeIter iter;
-
-                /* Append new iter */
-                gtk_tree_store_append (priv->store, &iter, current_iter);
-                book_tree_insert_node (tree, child, &iter, NULL);
-        }
-}
-
-static void
-book_tree_selection_changed_cb (GtkTreeSelection *selection,
-                                DhBookTree       *tree)
-{
-        DhBookTreePrivate *priv = dh_book_tree_get_instance_private (tree);
-        GtkTreeIter     iter;
-
-        if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
-                DhLink *link;
-
-                gtk_tree_model_get (GTK_TREE_MODEL (priv->store),
-                                    &iter,
-                                    COL_LINK, &link,
-                                    -1);
-                if (link) {
-                        if (link != priv->selected_link) {
-                                g_signal_emit (tree, signals[LINK_SELECTED], 0, link);
-                        }
-                        priv->selected_link = link;
-                }
-        }
-}
-
 static void
 book_tree_group_by_language_cb (GObject    *object,
                                 GParamSpec *pspec,
-                                gpointer    user_data)
+                                DhBookTree *tree)
 {
-        book_tree_populate_tree (DH_BOOK_TREE (user_data));
-}
-
-static void
-book_tree_init_selection (DhBookTree *tree)
-{
-        DhBookTreePrivate   *priv;
-        GtkTreeSelection *selection;
-        GtkTreeIter       iter;
-        gboolean          iter_found = FALSE;
-        DhLink           *link;
-
-        priv = dh_book_tree_get_instance_private (tree);
-
-        /* Mark the first item as selected, or it would get automatically
-         * selected when the treeview will get focus; but that's not even
-         * enough as a selection changed would still be emitted when there
-         * is no change, hence the manual tracking of selection in
-         * selected_link.
-         *   https://bugzilla.gnome.org/show_bug.cgi?id=492206
-         */
-        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
-        g_signal_handlers_block_by_func (selection,
-                                         book_tree_selection_changed_cb,
-                                         tree);
-
-        /* If grouping by languages, get first book in the first language */
-        if (dh_book_manager_get_group_by_language (priv->book_manager)) {
-                GtkTreeIter language_iter;
-
-                if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->store),
-                                                   &language_iter)) {
-                        iter_found = gtk_tree_model_iter_children (GTK_TREE_MODEL (priv->store),
-                                                                   &iter,
-                                                                   &language_iter);
-                }
-        } else {
-                iter_found = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->store),
-                                                            &iter);
-        }
-
-        if (iter_found) {
-                gtk_tree_model_get (GTK_TREE_MODEL (priv->store),
-                                    &iter,
-                                    COL_LINK, &link,
-                                    -1);
-                priv->selected_link = link;
-                gtk_tree_selection_select_iter (selection, &iter);
-        }
-
-        g_signal_handlers_unblock_by_func (selection,
-                                           book_tree_selection_changed_cb,
-                                           tree);
+        book_tree_populate_tree (tree);
 }
 
 static void
@@ -688,14 +690,14 @@ dh_book_tree_constructed (GObject *object)
                                  0);
 
         g_signal_connect_object (priv->book_manager,
-                                 "book-deleted",
-                                 G_CALLBACK (book_tree_book_deleted_or_disabled_cb),
+                                 "book-enabled",
+                                 G_CALLBACK (book_tree_book_created_or_enabled_cb),
                                  tree,
                                  0);
 
         g_signal_connect_object (priv->book_manager,
-                                 "book-enabled",
-                                 G_CALLBACK (book_tree_book_created_or_enabled_cb),
+                                 "book-deleted",
+                                 G_CALLBACK (book_tree_book_deleted_or_disabled_cb),
                                  tree,
                                  0);
 
@@ -713,8 +715,6 @@ dh_book_tree_constructed (GObject *object)
 
         book_tree_populate_tree (tree);
 
-        book_tree_init_selection (tree);
-
         G_OBJECT_CLASS (dh_book_tree_parent_class)->constructed (object);
 }
 
@@ -768,7 +768,8 @@ book_tree_find_uri_foreach (GtkTreeModel *model,
         gtk_tree_model_get (model, iter,
                             COL_LINK, &link,
                             -1);
-        if (link) {
+
+        if (link != NULL) {
                 gchar *link_uri;
 
                 link_uri = dh_link_get_uri (link);
@@ -779,6 +780,7 @@ book_tree_find_uri_foreach (GtkTreeModel *model,
                         data->iter = *iter;
                         data->path = gtk_tree_path_copy (path);
                 }
+
                 g_free (link_uri);
         }
 
@@ -792,6 +794,7 @@ dh_book_tree_select_uri (DhBookTree  *tree,
         DhBookTreePrivate   *priv = dh_book_tree_get_instance_private (tree);
         GtkTreeSelection *selection;
         FindURIData       data;
+        DhLink           *link;
 
         data.found = FALSE;
         data.uri = uri;
@@ -816,7 +819,14 @@ dh_book_tree_select_uri (DhBookTree  *tree,
                                          tree);
 
         gtk_tree_view_expand_to_path (GTK_TREE_VIEW (tree), data.path);
+
+        gtk_tree_model_get (GTK_TREE_MODEL (priv->store),
+                            &data.iter,
+                            COL_LINK, &link,
+                            -1);
+        priv->selected_link = link;
         gtk_tree_selection_select_iter (selection, &data.iter);
+
         gtk_tree_view_set_cursor (GTK_TREE_VIEW (tree), data.path, NULL, 0);
 
         g_signal_handlers_unblock_by_func (selection,
@@ -832,31 +842,31 @@ dh_book_tree_get_selected_book (DhBookTree *tree)
         GtkTreeSelection *selection;
         GtkTreeModel     *model;
         GtkTreeIter       iter;
-        GtkTreePath      *path;
-        DhLink           *link;
 
         selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
-        if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
+        if (!gtk_tree_selection_get_selected (selection, &model, &iter))
                 return NULL;
-        }
 
-        path = gtk_tree_model_get_path (model, &iter);
-
-        /* Get the book node for this link. */
+        /* Depending on whether books are grouped by language, the book link can
+         * be at a different depth. And it's safer to check that the returned
+         * link has the good type. So walk up the tree to find the book.
+         */
         while (TRUE) {
-                if (gtk_tree_path_get_depth (path) <= 1) {
-                        break;
-                }
+                DhLink *link;
+                GtkTreeIter parent;
 
-                gtk_tree_path_up (path);
+                gtk_tree_model_get (model, &iter,
+                                    COL_LINK, &link,
+                                    -1);
+
+                if (dh_link_get_link_type (link) == DH_LINK_TYPE_BOOK)
+                        return link;
+
+                if (!gtk_tree_model_iter_parent (model, &parent, &iter))
+                        break;
+
+                iter = parent;
         }
 
-        gtk_tree_model_get_iter (model, &iter, path);
-        gtk_tree_path_free (path);
-
-        gtk_tree_model_get (model, &iter,
-                            COL_LINK, &link,
-                            -1);
-
-        return link;
+        g_return_val_if_reached (NULL);
 }
