@@ -5,6 +5,7 @@
  * Copyright (C) 2005-2008 Imendio AB
  * Copyright (C) 2010 Lanedo GmbH
  * Copyright (C) 2013 Aleksander Morgado <aleksander@gnu.org>
+ * Copyright (C) 2015 Sébastien Wilmet <swilmet@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -30,21 +31,19 @@
 #include "dh-book-tree.h"
 
 typedef struct {
-        DhKeywordModel *model;
+        DhBookManager           *book_manager;
 
-        DhBookManager  *book_manager;
+        DhBookTree              *book_tree;
+        GtkScrolledWindow       *sw_book_tree;
 
-        DhLink         *selected_link;
+        GtkEntry                *entry;
+        DhKeywordModel          *hitlist_model;
+        GtkTreeView             *hitlist_view;
+        GtkScrolledWindow       *sw_hitlist;
 
-        GtkWidget      *entry;
-        GtkWidget      *hitlist;
-        GtkWidget      *sw_hitlist;
-        GtkWidget      *book_tree;
-        GtkWidget      *sw_book_tree;
-
-        GCompletion    *completion;
-        guint           idle_complete_id;
-        guint           idle_filter_id;
+        GCompletion             *completion;
+        guint                    idle_complete_id;
+        guint                    idle_filter_id;
 } DhSidebarPrivate;
 
 enum {
@@ -67,24 +66,35 @@ static gboolean
 sidebar_filter_idle_cb (DhSidebar *sidebar)
 {
         DhSidebarPrivate *priv;
-        const gchar *str;
-        DhLink      *link;
+        const gchar *search_text;
+        const gchar *book_id;
         DhLink      *book_link;
+        DhLink      *link;
 
         priv = dh_sidebar_get_instance_private (sidebar);
 
         priv->idle_filter_id = 0;
 
-        str = gtk_entry_get_text (GTK_ENTRY (priv->entry));
+        search_text = gtk_entry_get_text (priv->entry);
 
         book_link = dh_sidebar_get_selected_book (sidebar);
+        book_id = book_link != NULL ? dh_link_get_book_id (book_link) : NULL;
 
-        link = dh_keyword_model_filter (priv->model,
-                                        str,
-                                        book_link ? dh_link_get_book_id (book_link) : NULL,
+        /* Disconnect the model during the filter, for:
+         * 1. better performances.
+         * 2. clearing the selection.
+         */
+        gtk_tree_view_set_model (priv->hitlist_view, NULL);
+
+        link = dh_keyword_model_filter (priv->hitlist_model,
+                                        search_text,
+                                        book_id,
                                         NULL);
 
-        if (link)
+        gtk_tree_view_set_model (priv->hitlist_view,
+                                 GTK_TREE_MODEL (priv->hitlist_model));
+
+        if (link != NULL)
                 g_signal_emit (sidebar, signals[LINK_SELECTED], 0, link);
 
         return G_SOURCE_REMOVE;
@@ -109,11 +119,11 @@ sidebar_completion_add_book (DhSidebar *sidebar,
         GList *completions;
         DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
 
-        if (G_UNLIKELY (!priv->completion))
+        if (G_UNLIKELY (priv->completion == NULL))
                 priv->completion = g_completion_new (NULL);
 
         completions = dh_book_get_completions (book);
-        if (completions)
+        if (completions != NULL)
                 g_completion_add_items (priv->completion, completions);
 }
 
@@ -124,11 +134,11 @@ sidebar_completion_delete_book (DhSidebar *sidebar,
         GList *completions;
         DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
 
-        if (G_UNLIKELY (!priv->completion))
+        if (G_UNLIKELY (priv->completion == NULL))
                 return;
 
         completions = dh_book_get_completions (book);
-        if (completions)
+        if (completions != NULL)
                 g_completion_remove_items (priv->completion, completions);
 }
 
@@ -138,6 +148,7 @@ sidebar_book_created_or_enabled_cb (DhBookManager *book_manager,
                                     DhSidebar     *sidebar)
 {
         sidebar_completion_add_book (sidebar, book);
+
         /* Update current search if any */
         sidebar_search_run_idle (sidebar);
 }
@@ -148,6 +159,7 @@ sidebar_book_deleted_or_disabled_cb (DhBookManager *book_manager,
                                      DhSidebar     *sidebar)
 {
         sidebar_completion_delete_book (sidebar, book);
+
         /* Update current search if any */
         sidebar_search_run_idle (sidebar);
 }
@@ -156,20 +168,20 @@ static void
 sidebar_completion_populate (DhSidebar *sidebar)
 {
         DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
+        GList *books;
         GList *l;
 
-        for (l = dh_book_manager_get_books (priv->book_manager);
-             l;
-             l = g_list_next (l)) {
+        books = dh_book_manager_get_books (priv->book_manager);
+
+        for (l = books; l != NULL; l = l->next)
                 sidebar_completion_add_book (sidebar, DH_BOOK (l->data));
-        }
 }
 
 /******************************************************************************/
 
 static void
-sidebar_selection_changed_cb (GtkTreeSelection *selection,
-                              DhSidebar        *sidebar)
+sidebar_hitlist_selection_changed_cb (GtkTreeSelection *selection,
+                                      DhSidebar        *sidebar)
 {
         DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
         GtkTreeIter iter;
@@ -177,14 +189,11 @@ sidebar_selection_changed_cb (GtkTreeSelection *selection,
         if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
                 DhLink *link;
 
-                gtk_tree_model_get (GTK_TREE_MODEL (priv->model), &iter,
+                gtk_tree_model_get (GTK_TREE_MODEL (priv->hitlist_model), &iter,
                                     DH_KEYWORD_MODEL_COL_LINK, &link,
                                     -1);
 
-                if (link != priv->selected_link) {
-                        priv->selected_link = link;
-                        g_signal_emit (sidebar, signals[LINK_SELECTED], 0, link);
-                }
+                g_signal_emit (sidebar, signals[LINK_SELECTED], 0, link);
         }
 }
 
@@ -192,30 +201,28 @@ sidebar_selection_changed_cb (GtkTreeSelection *selection,
  * html view has been scrolled away.
  */
 static gboolean
-sidebar_tree_button_press_cb (GtkTreeView    *view,
-                              GdkEventButton *event,
-                              DhSidebar      *sidebar)
+sidebar_hitlist_button_press_cb (GtkTreeView    *hitlist_view,
+                                 GdkEventButton *event,
+                                 DhSidebar      *sidebar)
 {
         DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
         GtkTreePath  *path;
         GtkTreeIter   iter;
         DhLink       *link;
 
-        gtk_tree_view_get_path_at_pos (view, event->x, event->y, &path,
+        gtk_tree_view_get_path_at_pos (hitlist_view, event->x, event->y, &path,
                                        NULL, NULL, NULL);
-        if (!path)
+        if (path == NULL)
                 return GDK_EVENT_PROPAGATE;
 
-        gtk_tree_model_get_iter (GTK_TREE_MODEL (priv->model), &iter, path);
+        gtk_tree_model_get_iter (GTK_TREE_MODEL (priv->hitlist_model), &iter, path);
         gtk_tree_path_free (path);
 
-        gtk_tree_model_get (GTK_TREE_MODEL (priv->model),
+        gtk_tree_model_get (GTK_TREE_MODEL (priv->hitlist_model),
                             &iter,
                             DH_KEYWORD_MODEL_COL_LINK, &link,
                             -1);
 
-        priv->selected_link = link;
-
         g_signal_emit (sidebar, signals[LINK_SELECTED], 0, link);
 
         /* Always propagate the event so the tree view can update
@@ -233,7 +240,8 @@ sidebar_entry_key_press_event_cb (GtkEntry    *entry,
 
         if (event->keyval == GDK_KEY_Tab) {
                 if (event->state & GDK_CONTROL_MASK) {
-                        gtk_widget_grab_focus (priv->hitlist);
+                        if (gtk_widget_is_visible (GTK_WIDGET (priv->hitlist_view)))
+                                gtk_widget_grab_focus (GTK_WIDGET (priv->hitlist_view));
                 } else {
                         gtk_editable_set_position (GTK_EDITABLE (entry), -1);
                         gtk_editable_select_region (GTK_EDITABLE (entry), -1, -1);
@@ -249,14 +257,15 @@ sidebar_entry_key_press_event_cb (GtkEntry    *entry,
                 gchar       *name;
 
                 /* Get the first entry found. */
-                if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->model), &iter)) {
-                        gtk_tree_model_get (GTK_TREE_MODEL (priv->model),
+                if (gtk_widget_is_visible (GTK_WIDGET (priv->hitlist_view)) &&
+                    gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->hitlist_model), &iter)) {
+                        gtk_tree_model_get (GTK_TREE_MODEL (priv->hitlist_model),
                                             &iter,
                                             DH_KEYWORD_MODEL_COL_LINK, &link,
                                             DH_KEYWORD_MODEL_COL_NAME, &name,
                                             -1);
 
-                        gtk_entry_set_text (GTK_ENTRY (entry), name);
+                        gtk_entry_set_text (entry, name);
                         g_free (name);
 
                         gtk_editable_set_position (GTK_EDITABLE (entry), -1);
@@ -276,34 +285,34 @@ sidebar_entry_changed_cb (GtkEntry  *entry,
                           DhSidebar *sidebar)
 {
         DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
+        const gchar *search_text;
 
-        /* If search entry is empty, hide the hitlist */
-        if (strcmp (gtk_entry_get_text (entry), "") == 0) {
-                gtk_widget_hide (priv->sw_hitlist);
-                gtk_widget_show (priv->sw_book_tree);
-                return;
+        search_text = gtk_entry_get_text (entry);
+
+        if (search_text == NULL || search_text[0] == '\0') {
+                gtk_widget_hide (GTK_WIDGET (priv->sw_hitlist));
+                gtk_widget_show (GTK_WIDGET (priv->sw_book_tree));
+        } else {
+                gtk_widget_hide (GTK_WIDGET (priv->sw_book_tree));
+                gtk_widget_show (GTK_WIDGET (priv->sw_hitlist));
+                sidebar_search_run_idle (sidebar);
         }
-
-        gtk_widget_hide (priv->sw_book_tree);
-        gtk_widget_show (priv->sw_hitlist);
-        sidebar_search_run_idle (sidebar);
 }
 
 static gboolean
 sidebar_complete_idle_cb (DhSidebar *sidebar)
 {
         DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
-        const gchar  *str;
+        const gchar  *search_text;
         gchar        *completed = NULL;
-        gsize         length;
 
-        str = gtk_entry_get_text (GTK_ENTRY (priv->entry));
+        search_text = gtk_entry_get_text (priv->entry);
 
-        g_completion_complete (priv->completion, str, &completed);
-        if (completed) {
-                length = strlen (str);
+        g_completion_complete (priv->completion, search_text, &completed);
+        if (completed != NULL) {
+                gsize length = strlen (search_text);
 
-                gtk_entry_set_text (GTK_ENTRY (priv->entry), completed);
+                gtk_entry_set_text (priv->entry, completed);
                 gtk_editable_set_position (GTK_EDITABLE (priv->entry), length);
                 gtk_editable_select_region (GTK_EDITABLE (priv->entry),
                                             length, -1);
@@ -316,11 +325,11 @@ sidebar_complete_idle_cb (DhSidebar *sidebar)
 }
 
 static void
-sidebar_entry_text_inserted_cb (GtkEntry    *entry,
-                                const gchar *text,
-                                gint         length,
-                                gint        *position,
-                                DhSidebar   *sidebar)
+sidebar_entry_insert_text_cb (GtkEntry    *entry,
+                              const gchar *text,
+                              gint         length,
+                              gint        *position,
+                              DhSidebar   *sidebar)
 {
         DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
 
@@ -341,19 +350,9 @@ dh_sidebar_set_search_string (DhSidebar   *sidebar,
 
         priv = dh_sidebar_get_instance_private (sidebar);
 
-        g_signal_handlers_block_by_func (priv->entry,
-                                         sidebar_entry_changed_cb,
-                                         sidebar);
-
-        gtk_entry_set_text (GTK_ENTRY (priv->entry), str);
+        gtk_entry_set_text (priv->entry, str);
         gtk_editable_set_position (GTK_EDITABLE (priv->entry), -1);
         gtk_editable_select_region (GTK_EDITABLE (priv->entry), -1, -1);
-
-        g_signal_handlers_unblock_by_func (priv->entry,
-                                           sidebar_entry_changed_cb,
-                                           sidebar);
-
-        sidebar_search_run_idle (sidebar);
 }
 
 /******************************************************************************/
@@ -363,24 +362,24 @@ dh_sidebar_set_search_focus (DhSidebar *sidebar)
 {
         DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
 
-        gtk_widget_grab_focus (priv->entry);
+        gtk_widget_grab_focus (GTK_WIDGET (priv->entry));
 }
 
 /******************************************************************************/
 
 static void
-search_cell_data_func (GtkTreeViewColumn *tree_column,
-                       GtkCellRenderer   *cell,
-                       GtkTreeModel      *tree_model,
-                       GtkTreeIter       *iter,
-                       gpointer           data)
+hitlist_cell_data_func (GtkTreeViewColumn *tree_column,
+                        GtkCellRenderer   *cell,
+                        GtkTreeModel      *hitlist_model,
+                        GtkTreeIter       *iter,
+                        gpointer           data)
 {
         DhLink       *link;
         PangoStyle    style;
         PangoWeight   weight;
         gboolean      current_book_flag;
 
-        gtk_tree_model_get (tree_model, iter,
+        gtk_tree_model_get (hitlist_model, iter,
                             DH_KEYWORD_MODEL_COL_LINK, &link,
                             DH_KEYWORD_MODEL_COL_CURRENT_BOOK_FLAG, &current_book_flag,
                             -1);
@@ -409,16 +408,11 @@ search_cell_data_func (GtkTreeViewColumn *tree_column,
 /******************************************************************************/
 
 static void
-sidebar_book_tree_link_selected_cb (GObject   *ignored,
-                                    DhLink    *link,
-                                    DhSidebar *sidebar)
+sidebar_book_tree_link_selected_cb (DhBookTree *book_tree,
+                                    DhLink     *link,
+                                    DhSidebar  *sidebar)
 {
-        DhSidebarPrivate *priv = dh_sidebar_get_instance_private (sidebar);
-
-        if (link != priv->selected_link) {
-                priv->selected_link = link;
-                g_signal_emit (sidebar, signals[LINK_SELECTED], 0, link);
-        }
+        g_signal_emit (sidebar, signals[LINK_SELECTED], 0, link);
 }
 
 DhLink *
@@ -430,7 +424,7 @@ dh_sidebar_get_selected_book (DhSidebar *sidebar)
 
         priv = dh_sidebar_get_instance_private (sidebar);
 
-        return dh_book_tree_get_selected_book (DH_BOOK_TREE (priv->book_tree));
+        return dh_book_tree_get_selected_book (priv->book_tree);
 }
 
 void
@@ -443,7 +437,7 @@ dh_sidebar_select_uri (DhSidebar   *sidebar,
 
         priv = dh_sidebar_get_instance_private (sidebar);
 
-        dh_book_tree_select_uri (DH_BOOK_TREE (priv->book_tree), uri);
+        dh_book_tree_select_uri (priv->book_tree, uri);
 }
 
 /******************************************************************************/
@@ -463,6 +457,7 @@ dh_sidebar_dispose (GObject *object)
         DhSidebarPrivate *priv = dh_sidebar_get_instance_private (DH_SIDEBAR (object));
 
         g_clear_object (&priv->book_manager);
+        g_clear_object (&priv->hitlist_model);
 
         if (priv->idle_complete_id != 0) {
                 g_source_remove (priv->idle_complete_id);
@@ -492,72 +487,71 @@ dh_sidebar_init (DhSidebar *sidebar)
 {
         DhSidebarPrivate *priv;
         GtkCellRenderer  *cell;
-        GtkWidget        *hbox;
 
         priv = dh_sidebar_get_instance_private (sidebar);
 
-        /* Setup keyword model */
-        priv->model = dh_keyword_model_new ();
-
-        /* Setup hitlist */
-        priv->hitlist = gtk_tree_view_new ();
-        gtk_tree_view_set_model (GTK_TREE_VIEW (priv->hitlist), GTK_TREE_MODEL (priv->model));
-        gtk_tree_view_set_enable_search (GTK_TREE_VIEW (priv->hitlist), FALSE);
-
-        /* Setup the top-level box with entry search and Current|All buttons */
-        hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
-        gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
-        gtk_box_pack_start (GTK_BOX (sidebar), hbox, FALSE, FALSE, 0);
-
         /* Setup the search entry */
-        priv->entry = gtk_search_entry_new ();
-        gtk_box_pack_start (GTK_BOX (hbox), priv->entry, TRUE, TRUE, 0);
+        priv->entry = GTK_ENTRY (gtk_search_entry_new ());
+        gtk_widget_set_hexpand (GTK_WIDGET (priv->entry), TRUE);
+        g_object_set (priv->entry,
+                      "margin", 6,
+                      NULL);
+        gtk_box_pack_start (GTK_BOX (sidebar), GTK_WIDGET (priv->entry), FALSE, FALSE, 0);
+
         g_signal_connect (priv->entry, "key-press-event",
                           G_CALLBACK (sidebar_entry_key_press_event_cb),
                           sidebar);
-        g_signal_connect (priv->hitlist, "button-press-event",
-                          G_CALLBACK (sidebar_tree_button_press_cb),
-                          sidebar);
         g_signal_connect (priv->entry, "changed",
                           G_CALLBACK (sidebar_entry_changed_cb),
                           sidebar);
         g_signal_connect (priv->entry, "insert-text",
-                          G_CALLBACK (sidebar_entry_text_inserted_cb),
+                          G_CALLBACK (sidebar_entry_insert_text_cb),
+                          sidebar);
+
+        /* Setup hitlist */
+        priv->hitlist_model = dh_keyword_model_new ();
+        priv->hitlist_view = GTK_TREE_VIEW (gtk_tree_view_new ());
+        gtk_tree_view_set_model (priv->hitlist_view, GTK_TREE_MODEL (priv->hitlist_model));
+        gtk_tree_view_set_headers_visible (priv->hitlist_view, FALSE);
+        gtk_tree_view_set_enable_search (priv->hitlist_view, FALSE);
+        gtk_widget_show (GTK_WIDGET (priv->hitlist_view));
+
+        g_signal_connect (priv->hitlist_view, "button-press-event",
+                          G_CALLBACK (sidebar_hitlist_button_press_cb),
+                          sidebar);
+
+        g_signal_connect (gtk_tree_view_get_selection (priv->hitlist_view),
+                          "changed",
+                          G_CALLBACK (sidebar_hitlist_selection_changed_cb),
                           sidebar);
 
-        /* Setup the hitlist */
-        priv->sw_hitlist = gtk_scrolled_window_new (NULL, NULL);
-        gtk_widget_set_no_show_all (priv->sw_hitlist, TRUE);
-        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (priv->sw_hitlist),
-                                        GTK_POLICY_NEVER,
-                                        GTK_POLICY_AUTOMATIC);
         cell = gtk_cell_renderer_text_new ();
         g_object_set (cell,
                       "ellipsize", PANGO_ELLIPSIZE_END,
                       NULL);
-        gtk_tree_view_insert_column_with_data_func (
-                GTK_TREE_VIEW (priv->hitlist),
-                -1,
-                NULL,
-                cell,
-                search_cell_data_func,
-                sidebar,
-                NULL);
-        gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (priv->hitlist), FALSE);
-        gtk_tree_view_set_search_column (GTK_TREE_VIEW (priv->hitlist), DH_KEYWORD_MODEL_COL_NAME);
-        g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->hitlist)),
-                          "changed",
-                          G_CALLBACK (sidebar_selection_changed_cb),
-                          sidebar);
-        gtk_widget_show (priv->hitlist);
-        gtk_container_add (GTK_CONTAINER (priv->sw_hitlist), priv->hitlist);
-        gtk_box_pack_start (GTK_BOX (sidebar), priv->sw_hitlist, TRUE, TRUE, 0);
+        gtk_tree_view_insert_column_with_data_func (priv->hitlist_view,
+                                                    -1,
+                                                    NULL,
+                                                    cell,
+                                                    hitlist_cell_data_func,
+                                                    sidebar,
+                                                    NULL);
+
+        /* Hitlist packing */
+        priv->sw_hitlist = GTK_SCROLLED_WINDOW (gtk_scrolled_window_new (NULL, NULL));
+        gtk_widget_set_no_show_all (GTK_WIDGET (priv->sw_hitlist), TRUE);
+        gtk_scrolled_window_set_policy (priv->sw_hitlist,
+                                        GTK_POLICY_NEVER,
+                                        GTK_POLICY_AUTOMATIC);
+        gtk_container_add (GTK_CONTAINER (priv->sw_hitlist),
+                           GTK_WIDGET (priv->hitlist_view));
+        gtk_box_pack_start (GTK_BOX (sidebar), GTK_WIDGET (priv->sw_hitlist), TRUE, TRUE, 0);
 
         /* Setup the book tree */
-        priv->sw_book_tree = gtk_scrolled_window_new (NULL, NULL);
-        gtk_widget_show (priv->sw_book_tree);
-        gtk_widget_set_no_show_all (priv->sw_book_tree, TRUE);
-        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (priv->sw_book_tree),
+        priv->sw_book_tree = GTK_SCROLLED_WINDOW (gtk_scrolled_window_new (NULL, NULL));
+        gtk_widget_show (GTK_WIDGET (priv->sw_book_tree));
+        gtk_widget_set_no_show_all (GTK_WIDGET (priv->sw_book_tree), TRUE);
+        gtk_scrolled_window_set_policy (priv->sw_book_tree,
                                         GTK_POLICY_NEVER,
                                         GTK_POLICY_AUTOMATIC);
 
@@ -615,14 +609,14 @@ dh_sidebar_constructed (GObject *object)
                                  0);
 
         g_signal_connect_object (priv->book_manager,
-                                 "book-deleted",
-                                 G_CALLBACK (sidebar_book_deleted_or_disabled_cb),
+                                 "book-enabled",
+                                 G_CALLBACK (sidebar_book_created_or_enabled_cb),
                                  sidebar,
                                  0);
 
         g_signal_connect_object (priv->book_manager,
-                                 "book-enabled",
-                                 G_CALLBACK (sidebar_book_created_or_enabled_cb),
+                                 "book-deleted",
+                                 G_CALLBACK (sidebar_book_deleted_or_disabled_cb),
                                  sidebar,
                                  0);
 
@@ -632,18 +626,18 @@ dh_sidebar_constructed (GObject *object)
                                  sidebar,
                                  0);
 
-        priv->book_tree = dh_book_tree_new (priv->book_manager);
-        gtk_widget_show (priv->book_tree);
+        priv->book_tree = DH_BOOK_TREE (dh_book_tree_new (priv->book_manager));
+        gtk_widget_show (GTK_WIDGET (priv->book_tree));
         g_signal_connect (priv->book_tree,
                           "link-selected",
                           G_CALLBACK (sidebar_book_tree_link_selected_cb),
                           sidebar);
-        gtk_container_add (GTK_CONTAINER (priv->sw_book_tree), priv->book_tree);
-        gtk_box_pack_end (GTK_BOX (sidebar), priv->sw_book_tree, TRUE, TRUE, 0);
+        gtk_container_add (GTK_CONTAINER (priv->sw_book_tree), GTK_WIDGET (priv->book_tree));
+        gtk_box_pack_end (GTK_BOX (sidebar), GTK_WIDGET (priv->sw_book_tree), TRUE, TRUE, 0);
 
         sidebar_completion_populate (sidebar);
 
-        dh_keyword_model_set_words (priv->model, priv->book_manager);
+        dh_keyword_model_set_words (priv->hitlist_model, priv->book_manager);
 
         G_OBJECT_CLASS (dh_sidebar_parent_class)->constructed (object);
 }
